@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2012-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2013-2014 Google, Inc.
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
@@ -11,9 +10,15 @@
 # Copyright (c) 2016 Mateusz Bysiek <mb@mbdev.pl>
 # Copyright (c) 2017 Hugo <hugovk@users.noreply.github.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """Astroid hooks for the Python standard library."""
 
@@ -21,16 +26,27 @@ import functools
 import keyword
 from textwrap import dedent
 
-from astroid import MANAGER, UseInferenceDefault, inference_tip, InferenceError
-from astroid import arguments
-from astroid import exceptions
-from astroid import nodes
+from astroid import (
+    MANAGER,
+    InferenceError,
+    UseInferenceDefault,
+    arguments,
+    exceptions,
+    inference_tip,
+    nodes,
+    util,
+)
 from astroid.builder import AstroidBuilder, extract_node
-from astroid import util
-
 
 TYPING_NAMEDTUPLE_BASENAMES = {"NamedTuple", "typing.NamedTuple"}
-ENUM_BASE_NAMES = {"Enum", "IntEnum", "enum.Enum", "enum.IntEnum"}
+ENUM_BASE_NAMES = {
+    "Enum",
+    "IntEnum",
+    "enum.Enum",
+    "enum.IntEnum",
+    "IntFlag",
+    "enum.IntFlag",
+}
 
 
 def _infer_first(node, context):
@@ -40,15 +56,15 @@ def _infer_first(node, context):
         value = next(node.infer(context=context))
         if value is util.Uninferable:
             raise UseInferenceDefault()
-        else:
-            return value
-    except StopIteration:
-        raise InferenceError()
+        return value
+    except StopIteration as exc:
+        raise InferenceError from exc
 
 
 def _find_func_form_arguments(node, context):
-    def _extract_namedtuple_arg_or_keyword(position, key_name=None):
-
+    def _extract_namedtuple_arg_or_keyword(  # pylint: disable=inconsistent-return-statements
+        position, key_name=None
+    ):
         if len(args) > position:
             return _infer_first(args[position], context)
         if key_name and key_name in found_keywords:
@@ -79,7 +95,7 @@ def infer_func_form(node, base_type, context=None, enum=False):
         name, names = _find_func_form_arguments(node, context)
         try:
             attributes = names.value.replace(",", " ").split()
-        except AttributeError:
+        except AttributeError as exc:
             if not enum:
                 attributes = [
                     _infer_first(const, context).value for const in names.elts
@@ -108,11 +124,13 @@ def infer_func_form(node, base_type, context=None, enum=False):
                             _infer_first(const, context).value for const in names.elts
                         ]
                 else:
-                    raise AttributeError
+                    raise AttributeError from exc
                 if not attributes:
-                    raise AttributeError
-    except (AttributeError, exceptions.InferenceError):
-        raise UseInferenceDefault()
+                    raise AttributeError from exc
+    except (AttributeError, exceptions.InferenceError) as exc:
+        raise UseInferenceDefault from exc
+
+    attributes = [attr for attr in attributes if " " not in attr]
 
     # If we can't infer the name of the class, don't crash, up to this point
     # we know it is a namedtuple anyway.
@@ -160,7 +178,7 @@ def infer_named_tuple(node, context=None):
     class_node, name, attributes = infer_func_form(
         node, tuple_base_name, context=context
     )
-    call_site = arguments.CallSite.from_call(node)
+    call_site = arguments.CallSite.from_call(node, context=context)
     func = next(extract_node("import collections; collections.namedtuple").infer())
     try:
         rename = next(call_site.infer_argument(func, "rename", context)).bool_value()
@@ -170,8 +188,7 @@ def infer_named_tuple(node, context=None):
     if rename:
         attributes = _get_renamed_namedtuple_attributes(attributes)
 
-    replace_args = ", ".join("{arg}=None".format(arg=arg) for arg in attributes)
-
+    replace_args = ", ".join(f"{arg}=None" for arg in attributes)
     field_def = (
         "    {name} = property(lambda self: self[{index:d}], "
         "doc='Alias for field number {index:d}')"
@@ -246,6 +263,11 @@ def infer_enum(node, context=None):
                 name = ''
                 value = 0
             return [EnumAttribute()]
+        def __reversed__(self):
+            class EnumAttribute(object):
+                name = ''
+                value = 0
+            return (EnumAttribute, )
         def __next__(self):
             return next(iter(self))
         def __getitem__(self, attr):
@@ -265,6 +287,24 @@ def infer_enum(node, context=None):
     return iter([class_node.instantiate_class()])
 
 
+INT_FLAG_ADDITION_METHODS = """
+    def __or__(self, other):
+        return {name}(self.value | other.value)
+    def __and__(self, other):
+        return {name}(self.value & other.value)
+    def __xor__(self, other):
+        return {name}(self.value ^ other.value)
+    def __add__(self, other):
+        return {name}(self.value + other.value)
+    def __div__(self, other):
+        return {name}(self.value / other.value)
+    def __invert__(self):
+        return {name}(~self.value)
+    def __mul__(self, other):
+        return {name}(self.value * other.value)
+"""
+
+
 def infer_enum_class(node):
     """ Specific inference for enums. """
     for basename in node.basenames:
@@ -279,7 +319,6 @@ def infer_enum_class(node):
             if any(not isinstance(value, nodes.AssignName) for value in values):
                 continue
 
-            targets = []
             stmt = values[0].statement()
             if isinstance(stmt, nodes.Assign):
                 if isinstance(stmt.targets[0], nodes.Tuple):
@@ -288,16 +327,23 @@ def infer_enum_class(node):
                     targets = stmt.targets
             elif isinstance(stmt, nodes.AnnAssign):
                 targets = [stmt.target]
+            else:
+                continue
 
             inferred_return_value = None
-            if isinstance(stmt.value, nodes.Const):
-                if isinstance(stmt.value.value, str):
-                    inferred_return_value = '"{}"'.format(stmt.value.value)
+            if isinstance(stmt, nodes.Assign):
+                if isinstance(stmt.value, nodes.Const):
+                    if isinstance(stmt.value.value, str):
+                        inferred_return_value = repr(stmt.value.value)
+                    else:
+                        inferred_return_value = stmt.value.value
                 else:
-                    inferred_return_value = stmt.value.value
+                    inferred_return_value = stmt.value.as_string()
 
             new_targets = []
             for target in targets:
+                if isinstance(target, nodes.Starred):
+                    continue
                 # Replace all the assignments with our mocked class.
                 classdef = dedent(
                     """
@@ -307,13 +353,20 @@ def infer_enum_class(node):
                         return {return_value}
                     @property
                     def name(self):
-                        return {name}
+                        return "{name}"
                 """.format(
                         name=target.name,
                         types=", ".join(node.basenames),
                         return_value=inferred_return_value,
                     )
                 )
+                if "IntFlag" in basename:
+                    # Alright, we need to add some additional methods.
+                    # Unfortunately we still can't infer the resulting objects as
+                    # Enum members, but once we'll be able to do that, the following
+                    # should result in some nice symbolic execution
+                    classdef += INT_FLAG_ADDITION_METHODS.format(name=target.name)
+
                 fake = AstroidBuilder(MANAGER).string_build(classdef)[target.name]
                 fake.parent = target.parent
                 for method in node.mymethods():
@@ -342,17 +395,41 @@ def infer_typing_namedtuple_class(class_node, context=None):
     generated_class_node = next(infer_named_tuple(node, context))
     for method in class_node.mymethods():
         generated_class_node.locals[method.name] = [method]
+
+    for assign in class_node.body:
+        if not isinstance(assign, nodes.Assign):
+            continue
+
+        for target in assign.targets:
+            attr = target.name
+            generated_class_node.locals[attr] = class_node.locals[attr]
+
     return iter((generated_class_node,))
 
 
+def infer_typing_namedtuple_function(node, context=None):
+    """
+    Starting with python3.9, NamedTuple is a function of the typing module.
+    The class NamedTuple is build dynamically through a call to `type` during
+    initialization of the `_NamedTuple` variable.
+    """
+    klass = extract_node(
+        """
+        from typing import _NamedTuple
+        _NamedTuple
+        """
+    )
+    return klass.infer(context)
+
+
 def infer_typing_namedtuple(node, context=None):
     """Infer a typing.NamedTuple(...) call."""
     # This is essentially a namedtuple with different arguments
     # so we extract the args and infer a named tuple.
     try:
         func = next(node.func.infer())
-    except InferenceError:
-        raise UseInferenceDefault
+    except InferenceError as exc:
+        raise UseInferenceDefault from exc
 
     if func.qname() != "typing.NamedTuple":
         raise UseInferenceDefault
@@ -372,10 +449,11 @@ def infer_typing_namedtuple(node, context=None):
         names.append(elt.elts[0].as_string())
 
     typename = node.args[0].as_string()
-    node = extract_node(
-        "namedtuple(%(typename)s, (%(fields)s,)) "
-        % {"typename": typename, "fields": ",".join(names)}
-    )
+    if names:
+        field_names = "({},)".format(",".join(names))
+    else:
+        field_names = "''"
+    node = extract_node(f"namedtuple({typename}, {field_names})")
     return infer_named_tuple(node, context)
 
 
@@ -393,6 +471,12 @@ MANAGER.register_transform(
 MANAGER.register_transform(
     nodes.ClassDef, inference_tip(infer_typing_namedtuple_class), _has_namedtuple_base
 )
+MANAGER.register_transform(
+    nodes.FunctionDef,
+    inference_tip(infer_typing_namedtuple_function),
+    lambda node: node.name == "NamedTuple"
+    and getattr(node.root(), "name", None) == "typing",
+)
 MANAGER.register_transform(
     nodes.Call, inference_tip(infer_typing_namedtuple), _looks_like_typing_namedtuple
 )
